
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; ===========
          ; HP_DESK2.IS
          ; ===========
          ;
          ; Calamus printer driver for HP Laserjet+, HP Deskjet
          ;
          ;
          ; Autor     : DMC,
          ;             WL
          ;
          ; Version   : 2.1
          ;
          ;             print data optimizing with horizontal
          ;             offset, data width and vertical skip
          ;
          ; Date      : 02.06.1989
          ;
          ; Assembler :
          ;  GFA Assembler 1.1
          ;
          ; Disassembler :
          ;  < Detective >    Copyright 1986 FOCUS Computer Gmbh
          ;  Autor:  Detlev Ponnet                   Version 1.0
          ;
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


          .MACRO pushall
          movem.l   d1-d7/a0-a6,-(sp)
          .ENDM 

          .MACRO popall
          movem.l   (sp)+,d1-d7/a0-a6
          .ENDM 

          .MACRO illegal
          .DC.w $4afc
          .ENDM 


          .TEXT 

          .DC.b 'CALAMUS'
          .DC.b 'CPD'         ; calamus printer driver

          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; *** MAIN ENTRY POINT ***
          ;
          ;         jsr cpd_entry
          ;
          ;
          ;
          ; stack frame on entry :
          ; ----------------------
          ;
          ;
          ;   14(a6)     .l    move down # pixels
          ;   10(a6)     .l    pointer to info record ( see below )
          ;    8(a6)     .w    function = offset in jump table
          ;
          ;
          ; info record :
          ; -------------
          ;
          ;     $26
          ;     $22
          ;     $20
          ;     $1E  .w  offset in dpi table ( 0..75, 1..100, 2..150, 3..300 )
          ;     $18  .l  vertical offset in 1/720 inches
          ;     $14  .l  horizontal offset in 1/720 inches
          ;     $12  .w  number of lines
          ;     $10  .w
          ;     $0C  .l  length of data block to print = one line
          ;       8
          ;       4
          ;       0  .l  adr of data block to print ( all lines )
          ;
          ;
          ;
          ; CALL SEQUENCE :
          ; ---------------
          ;
          ; 0
          ;   1
          ; 9
          ;   1
          ; 9
          ;   1
          ;   1
          ;   1
          ; 2       ; INIT CODES
          ;   1
          ;   1
          ; 4       ; MAIN DATA
          ; 3       ; END CODES
          ;   1
          ; 9
          ;   1
          ;   1
          ;
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

cpd_entry:          
          link      a6,#0
          movem.l   d1-d7/a0-a5,-(sp)

          ; hardware register
          movea.w   #$8800,a2 ; FFFF 8800
          movea.w   #$fa01,a3 ; FFFF FA01

          clr.l     d0
          lea.l     L10036,a0
          move.w    8(a6),d1  ; function
          lsl.w     #2,d1
          adda.w    d1,a0
          movea.l   (a0),a0

          ;................................................
          ;
          ; prints ####xxxx id for every call of printer driver
          ;

debug     = 0

          .IF debug
          pushall 

          ext.l     d1
          move.l    d1,d0
          lea.l     debugtext,a0
          bsr       number_to_ascii

          lea.l     debugcodes,a0
          bsr       print_block

          popall 
          clr.l     d0
          .ENDIF 
          ;...............................................

          jmp       (a0)      ; jump indirect to function


          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; exit
          ;
          ; -> D0 ... status ( or ptr )
          ;

exit:     movem.l   (sp)+,d1-d7/a0-a5
          unlk      a6
          rts       



endrg:    
          pushall 

          lea.l     L1048B,a0 ; end raster graphics
          bsr       print_block

          popall 
          rts       


L10036:   
          .DC.l exit          ; 0 BEGIN PAGE
          .DC.l L10062        ; 1 get ptr to printer characteristics data block
          .DC.l L1006a        ; 2 init printer, start graphics
          .DC.l L100d6        ; 3 exit printer, end graphics
          .DC.l L100e4        ; 4 transfer lines
          .DC.l exit          ; 5
          .DC.l exit          ; 6
          .DC.l L101ca        ; 7 move down # pixels
          .DC.l L101b2        ; 8 (transfer block)
          .DC.l L101e6        ; 9 NOP
          .DC.l exit          ;10


L10062:   move.l    #L1027e,d0          ; printer characteristics
          bra.s     exit

L1006a:   
          movea.l   $a(a6),a1
          move.w    $22(a1),L10498

          move.l    $14(a1),d0          ; number
          lea.l     L1045f,a0 ; move current print position to decipoint
          bsr       number_to_ascii

          move.l    $18(a1),d0
          lea.l     L10467,a0 ; vertical offset
          bsr       number_to_ascii

          move.w    $1e(a1),d0          ;  offset in dpi table


          ;...............................................................
          ; line skip with ESC * p command must be converted
          ; for different resolutions

          move.w    #4,RESOLUTION
          sub.w     d0,RESOLUTION       ; 0..75, 1..100, 2..150, 3..300

          ; -> convert factor           ; 4      3       2       1
          ;...............................................................


          lsl.w     #2,d0     ; * 4
          lea.l     L103F0,a0 ; base adr of dpi table
          move.w    0(a0,d0.w),d0       ; get dpi value
          lea.l     L10457,a0 ; set dpi
          bsr       number_to_ascii

          bsr       endempty  ; FF ?

          lea.l     L10448,a0 ; init escape codes
          ;    clr.l     d0
          ;    move.w    $26(a1),d0
          ;    divu      #10,d0
          ;    addi.w    #'0',d0
          ;    move.b    d0,9(a0)
          ;    swap.w    d0
          ;    addi.w    #'0',d0
          ;    move.b    d0,10(a0)
          bsr       print_block
          bra       exit


          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; end raster graphics = end of page
          ;
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

L100d6:   
          bsr       endrg
          bsr       endempty
          bra       exit


endempty: tst.l     EMPTYNR
          beq       noff

          moveq.l   #$c,d0    ; form feed
          bsr       druck
          bsr       setnoerr  ; status ok for CALAMUS
noff:     
          clr.l     EMPTYNR   ; skip blank lines counter
          rts       


          ; no error
setnoerr: moveq.l   #0,d0
          tst.l     d0
          rts       


L100e4:   
          movea.l   $a(a6),a1

          movea.l   (a1),a4   ; adr of block  to print

          move.l    $c(a1),d7 ; length of block

          move.w    $12(a1),d6
          subq.w    #1,d6     ; # of lines

          ;    move.l    d7,d0
          ;    lea.l     L1047c,a0 ; ESC * b 0 0 0 0 W
          ;    bsr       number_to_ascii
          subq.w    #1,d7
          clr.w     d5

          ; next line
L10104:   
          ; print data block
          ;
          ; -> D7 .. len
          ;    A4 .. adr

          pushall 
          movea.l   a4,a0     ; address
          move.l    d7,d1     ; D7 must be constant for next line
          addq.l    #1,d1
          bsr       DoOptimize
          popall 

          tst.w     d0
          beq       Error

          addq.l    #1,a4
          adda.l    d7,a4     ; move buffer pointer

          bsr       setnoerr
          ; no error
          moveq.l   #0,d0
          tst.w     d0

          dbf       d5,L10184

          moveq.l   #$14,d5
          movem.l   d1-d7/a0-a6,-(sp)

          move.w    #2,-(sp)  ; CON:
          move.w    #1,-(sp)  ;bconstat
          trap      #$d       ;BIOS
          addq.w    #4,sp
          tst.l     d0
          beq       L10156    ; kein Zeichen von Tastatur verfgbar

          move.w    #2,-(sp)  ; CON:
          move.w    #2,-(sp)  ;conin
          trap      #$d       ;BIOS     Zeichen von CON: lesen
          addq.w    #4,sp
          cmpi.l    #$610000,d0         ; UNDO ?
          beq       L10156    ; ja

          clr.l     d0
L10156:   movem.l   (sp)+,d1-d7/a0-a6
          tst.l     d0
          beq       L10184

          ; ********************
          ; *** UNDO pressed ***
          ; ********************

          bsr       L101A8
          move.w    #1,UndoFlag

          bsr       endrg

          clr.l     d0
          move.w    #1,d0
          bset.l    #$1f,d0   ; UNDO = $10000001

          bra       exit

L10184:   dbne      d6,L10104 ; next line
          bra       exit


Error:    
          bsr       timeout   ;;

          clr.w     UndoFlag
          bsr       L101A8

          move.l    d0,-(sp)  ; save error status
          bsr       endrg

          bsr       endempty

          move.l    (sp)+,d0

          bra       exit


L101A8:   movem.l   d0-d7/a0-a5,L1049A
          rts       


L101b2:   movem.l   L1049A,d0-d7/a0-a5

          tst.w     UndoFlag
          bne       L10104    ; ja, undo

          subq.l    #1,a4
          bra       L10104



          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; move down # pixels
          ;
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

L101ca:   move.l    $e(a6),d0
          lea.l     L10486,a0
          bsr       number_to_ascii
          lea.l     L10481,a0 ; move down # of pixels
          bsr       print_block
          bra       exit


L101e6:   clr.l     d0
          move.w    #$ffff,d0
          bra       exit



          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; print block
          ;
          ; -> a0 ... points to data block
          ;           first byte in data block = length of block
          ;
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print_block:        
          clr.l     d3        ;;.w
          move.b    (a0)+,d3  ; # of bytes to print

          move.l    a0,-(sp)  ; address
          addq.l    #1,d3
          move.l    d3,-(sp)  ; count
          bsr       dospool
          addq.l    #8,sp
          tst.w     d0
          beq       spoolerror

          ; no error

          moveq.l   #0,d0
          tst.w     d0
          rts       

spoolerror:         
          bsr       timeout
          rts       



          ; *** printer timeout ***
timeout:  
          clr.l     d0
          move.w    #$fffc,d0
          bset.l    #$1f,d0
          tst.w     d0

          rts       




          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; convert number in D0 to ascii
          ;
          ; -> D0 .. number
          ;    A0 .. address to store ascii string
          ;
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

number_to_ascii:    
          move.l    #1000,d1
L10268:   
          divu      d1,d0
          addi.w    #'0',d0
          move.b    d0,(a0)+
          clr.w     d0
          swap.w    d0
          divu      #10,d1
          tst.w     d1
          bne.s     L10268
          rts       



          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; dospool
          ;
          ; transfers count bytes starting at address adr to printer
          ; via extended SPLX functions,
          ; or standard $50a vector call if SPLX is not installed
          ;
          ;
          ; CALL SYNTAX :
          ;
          ;   move.l    adr,-(sp)  ; address
          ;   move.l    count,-(sp)         ; count
          ;   bsr       dospool
          ;   addq.l    #8,sp
          ;   tst.w     d0
          ;   beq       spoolerror
          ;
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

adr       = 12
len       = 8
dospool:  
          link      a6,#0
          movem.l   d1-d7/a0-a5,-(sp)   ; save regs

          bsr       testspooler
          bne       nospooler

          move.l    adr(a6),-(sp)       ; adr
          move.l    len(a6),-(sp)       ; len
          move.l    #'SPLB',-(sp)
          suba.l    a5,a5
          movea.l   $50a(a5),a0         ; vektor send char to printer
          ;;                              (parallel port)
          jsr       (a0)
          adda.l    #12,sp
          tst.w     d0
          bne       spoolok

          ; spool not ok, send bytes individually

nospooler:          
          move.l    len(a6),d1
          subq.l    #1,d1
          movea.l   adr(a6),a3
n_byte:   
          move.b    (a3)+,d0  ; next byte
          bsr       druck
          beq       printerr

          dbra      d1,n_byte ; next byte
printerr: 
spoolok:  
          movem.l   (sp)+,d1-d7/a0-a5   ; restore regs
          unlk      a6
          rts       


druck:              ;prints char in d0
          movem.l   d1-d7/a0-a6,-(sp)
          ext.w     d0
          move.w    d0,-(sp)
          move.w    d0,-(sp)
          suba.l    a5,a5
          movea.l   $50a(a5),a0         ; vektor send char to printer
          ;;                              (parallel port)
          jsr       (a0)
          addq.l    #4,sp
          movem.l   (sp)+,d1-d7/a0-a6
          tst.w     d0
          rts       


          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; test if extended SPLX functions available
          ;
          ; must be in supervisor mode for this call
          ;
          ;
          ; bsr setspooler
          ; beq spooler_available
          ;
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

testspooler:        
          movea.l   $50a,a0   ; supervisor access
          cmpi.l    #'XBRA',-12(a0)
          bne       nospl
          cmpi.l    #'SPLX',-8(a0)
nospl:    rts       


          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



          ;****************************************************************



          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; DoOptimize
          ; ----------
          ;
          ; searches non zero bytes in buffer and sends ESC * B # x # W
          ; and data bytes to printer.
          ;
          ; - skips zero bytes at begin and end of line
          ; - skips empty lines
          ;
          ;
          ; -> A0 ... buffer adr
          ;    D1 ... length
          ;
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DoOptimize:         
          pushall 
          bsr       opti1
          popall 
          rts       


opti1:    
          move.l    a0,VSTARTADR
          move.l    d1,LINESIZE

          ; search first non empty byte
          ; => set temporary X offset ESC * b # X

          subq.l    #1,d1
next1:    move.b    (a0)+,d0
          tst.b     d0
          bne       nonzero1  ; line is not empty
          dbf       d1,next1

          ; line is empty <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
isempty:  
          ;          illegal

          addq.l    #1,EMPTYNR
          moveq.l   #-1,d0
          rts       

          ; LINE IS NOT EMPTY <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
nonzero1: 
          ;          illegal

          bsr       doempty   ; handle pending empty lines

          suba.l    VSTARTADR,a0        ; startwert abziehen => byte offset
          subq.l    #1,a0
          move.l    a0,d4

          ; => D4 ... horizontal offset in bytes

          move.l    d4,VLINEXOFF        ; byte horizontal offset
          asl.l     #3,d4     ; *8 : bytes => bits
          movea.l   #LINEXOFF,a6
          move.w    #4-1,d1   ; 4 Ziffern schreiben
          bsr       d4todec   ; convert to 4 byte ASCII

          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; search end of line
          ; => set graphics line width
          ;
          movea.l   VSTARTADR,a0
          adda.l    LINESIZE,a0

          move.l    LINESIZE,d1
          subq.l    #1,d1
next2:    move.b    -(a0),d0
          tst.b     d0
          bne       nonzero2
          dbf       d1,next2
nonzero2: 
          addq.l    #1,a0

          suba.l    VSTARTADR,a0        ; startwert abziehen => byte offset
          suba.l    VLINEXOFF,a0        ; horiz offset abziehen
          move.l    a0,d4

          ; => D4 ... line width in bytes

          move.l    d4,VLINEWID
          movea.l   #LINEWID,a6
          move.w    #3-1,d1   ; 3 Ziffern schreiben
          bsr       d4todec   ; convert to ASCII

sendinit: 
          pushall 
          movea.l   #INITLINE,a0        ; start raster graphics line
          bsr       print_block         ; send to printer
          popall 

          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; bytes an Drucker senden
          ;
sendbytes:          
          movea.l   VSTARTADR,a0
          adda.l    VLINEXOFF,a0

          move.l    VLINEWID,d1         ; variable
          subq.l    #1,d1

next4:    
          move.l    a0,-(sp)  ; adr
          ext.l     d1
          addq.l    #1,d1
          move.l    d1,-(sp)  ; len
          bsr       dospool
          addq.l    #8,sp
          tst.w     d0
          rts       




          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; start of subroutines
          ;
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

          ;**************************
          ; mkbytstr: d4.b in String wandeln
          ;**************************
mkbytstr: 
          move.w    #2,d1     ; 3 Ziffern schreiben


          ;*************************
          ; d4 to decimal string
          ; IN: Stringadresse in a6
          ;        Anzahl der zu wandelnden Ziffern-1 in d1
          ;*************************
d4todec:  
          movem.l   d1/d2/d4,-(sp)      ; Register retten
nxtdigit: 
          move.l    d4,d2     ; d4 nach d2
          divu      #10,d2    ; durch 10
          move.w    d2,d4     ; Quotient nach d4.w
          swap.w    d2        ; Rest nach d2.w
          addi.b    #48,d2    ; in ASCII wandeln
          move.b    d2,0(a6,d1.w)       ; in String kopieren
          dbf       d1,nxtdigit         ; nchste Ziffer

          movem.l   (sp)+,d1/d2/d4      ; Register holen
          rts                 ; raus

          ;*************************
          ; d4 to hexadecimal string
          ; IN: Stringadresse in a6
          ;        Anzahl der zu wandelnden Nibbles-1 in d1
          ;*************************
d4tohex:  
          movem.l   d1/d2/d4,-(sp)      ; Register retten
nextbyte: 
          move.l    d4,d2     ; d4 nach d2
          asr.l     #4,d4     ; 1 Nibble weiter
          andi.l    #$0000000f,d2       ; unterstes Nibble ausmaskieren
          cmpi.b    #9,d2     ; Zahl oder Buchstabe?
          bls.s     numeric   ; Zahl, dann weiter
          addi.b    #7,d2     ; Buchstabe, plus 7
numeric:  
          addi.b    #48,d2    ; in ASCII wandeln
          move.b    d2,0(a6,d1.w)       ; in String kopieren
          dbf       d1,nextbyte         ; nchstes Byte

          movem.l   (sp)+,d1/d2/d4      ; Register holen
          rts                 ; raus









          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;
          ; HANDLE EMPTY LINES
          ;
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
doempty:  
          pushall 

          move.l    EMPTYNR,d2          ; are there empty lines to print ?
          tst.l     d2
          beq       econt     ; no empty lines, done

          cmpi.l    #4,d2
          bls       NOVOFF    ; noch nicht genug Leerzeilen fr V offset

          ; send VOFF command
          move.l    d2,d4
          mulu.w    RESOLUTION,d4       ; convert for RESOLUTION

          movea.l   #SKIPNR,a6
          move.w    #4-1,d1   ; 4 Ziffern schreiben
          bsr       d4todec   ; convert to 4 byte ASCII

          movea.l   #SKIPV,a0 ; skip vertical
          bsr       print_block         ; send to printer
          bra       econt
NOVOFF:   
          subq.l    #1,d2     ; for dbf
empty2:   
          pushall 
          movea.l   #EMPTYLINE,a0       ; blank line
          bsr       print_block         ; send to printer
          popall 
          dbf       d2,empty2 ; max. 4 empty lines
econt:    
          clr.l     EMPTYNR   ; done
          popall 
          rts       





          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

          .DATA 

          .EVEN 

ESC       = $1b


          .IF debug
debugcodes:         
          .DC.b e0-debugcodes-2
          .DC.b "####"
debugtext:          
          .DC.b 0,0,0,0
e0:       
          .ENDIF 

          .EVEN 

LINESIZE: .DC.l 0
VLINEWID: .DC.l 0
VWID:     .DC.l 0   ; line width before compression
VLINEXOFF:          .DC.l 0
VSTARTADR:          .DC.l 0
VLINECOUNT:         .DC.l 0

SAVESTACK:          .DC.l 0
VABBRUCH: .DC.w 0
VFABORT:  .DC.l 0

          .DC.b 0
INITLINE: 
          .DC.b E2-INITLINE-2
          .DC.b ESC,'*b'

LINEXOFF: .DC.b '0000'        ; horizontal line offset
          .DC.b 'x'

LINEWID:  .DC.b '300W'        ; line width
E2:       

          .EVEN 
          .DC.b 0
EMPTYLINE:          
          .DC.b E3-EMPTYLINE-2
          .DC.b ESC,'*b1W',0  ; empty line
E3:       

EMPTYNR:  .DC.l 0

SKIPV:    
          .DC.b ESKIPV-SKIPV-2
          .DC.b ESC,'*p+'
SKIPNR:   .DC.b '0001Y'
ESKIPV:   


RESOLUTION:         .DC.w 0


L1027e:   
          .DC.w 1000
          .DC.w $10
          .DC.w $3a
          .DC.w 99

          .DC.w 6   ; 6 paper sizes
          .DC.l L102b8

          .DC.w 4   ; 4 resolutions : 75 / 100 / 150 / 300
          .DC.l L103F0

          .DC.w 3   ; 3 modes : Kassette / Einzelblatt / Briefumschlag
          .DC.l L10400

          .DC.l 0

          .DC.b 'HP Deskjet  (Spool V2.1)',0
          ;      --------------------------------             ;max len
          .DC.b 0


          ;-------------------------------------------------------
          ;
          ; paper sizes
          ;
L102b8:   
          ; A5                            14.85 cm x 21.00 cm
          .DC.w 0,0,0,0,0,0,0,$0,$0,$0,$0,$1071,$0,$1741,0,0,0,0,8,8,1,1,0,0,0,0

          ; A4                            21.00 cm x 29.70 cm
          .DC.w 0,1,0,0,0,0,0,$dc,0,$171,0,$1651,0,$1ed6,0,0,0,0,8,8,1,1,0,0,0,0

          ; B5                            17.60 cm x 25.00 cm
          .DC.w 0,3,0,0,0,0,0,$0,$0,$0,$0,$137d,$0,$1baf,0,0,0,0,8,8,1,1,0,0,0,0

          ; Letter                        21.59 cm x 27.94 cm
          .DC.w 0,4,0,0,0,0,0,$0,$0,$0,$0,$17e8,$0,$1ef0,0,0,0,0,8,8,1,1,0,0,0,0

          ; Legal                         21.59 cm x 35.56 cm
          .DC.w 0,5,0,0,0,0,0,$0,$0,$0,$0,$17e8,$0,$2760,0,0,0,0,8,8,1,1,0,0,0,0

          ; Half                          13.97 cm x 21.59 cm
          .DC.w 0,7,0,0,0,0,0,$0,$0,$0,$0,$0f78,$0,$17e8,0,0,0,0,8,8,1,1,0,0,0,0


          ;-------------------------------------------------------

L103F0:             ;      ; DPI x DPI
          .DC.w 75,75
          .DC.w 100,100
          .DC.w 150,150
          .DC.w 300,300

          ;-------------------------------------------------------

L10400:   .DC.w 0
          .DC.l L10412

          .DC.w 1
          .DC.l L1041b

          .DC.w 0
          .DC.l L10430

L10412:   .DC.b 'Kassette',0
L1041b:   .DC.b 'Einzelblattzufhrung',0
L10430:   .DC.b 'Briefumschlag-Zufhrung',0


          ;----------------------------------------
L10448:   
          .DC.b e1-L10448-2   ; len

          ;   .DC.b ESC,'&l0O'    ; *** UNDEFINED FOR DESKJET ***

          ;   .DC.b ESC,'&l'
          ;   .DC.b 0,0,'X'       ; *** UNDEFINED FOR DESKJET ***

          .DC.b ESC,'*t'
L10457:   .DC.b 0,0,0,0,'R'   ; SET DPI

          .DC.b ESC,'&a'
L1045f:   .DC.b 0,0,0,0,'H'   ; HORIZONTAL POSITION

          .DC.b ESC,'&a'
L10467:   .DC.b 0,0,0,0,'V'   ; VERTICAL POSITION

          .DC.b ESC,'*r1A'    ; START RASTER GRAPHICS AT CURRENT POS
e1:       

          ;-------------------------------------------

L10471:   .DC.b e2-L10471-2   ; len

          .DC.b ESC,'*rB'     ; END RASTER GRAPHICS
          .DC.b ESC,'E'       ; RESET
e2:       

          ;-------------------------------------------

L10478:   .DC.b e3-L10478-2   ; len

          .DC.b ESC,'*b'
L1047c:   .DC.b 0,0,0,0,'W'   ; TRANSFER GRAPHICS BYTES
e3:       

          ;-------------------------------------------

L10481:   .DC.b e4-L10481-2   ; len

          .DC.b ESC,'*p+'
L10486:   .DC.b 0,0,0,0,'Y'   ; MOVE DOWN # PIXELS
e4:       

          ;-------------------------------------------

L1048B:   .DC.b e5-L1048B-2   ; len

          .DC.b ESC,'*rB'     ; END RASTER GRAPHICS
          ;        .DC.b ESC,'&l0X'    ; *** UNDEFINED FOR DESKJET ***
          ;        .DC.b ESC,'-',0     ; *** UNDEFINED FOR DESKJET ***
e5:       

          ;-------------------------------------------


          ;; Detective -- Datenbereich ;;;;;;;;;;;;;;;;;;;;;;;;;;;

          .BSS 

L10498:   .DS.w 1

L1049A:   .DS.b $40 ; register save area

UndoFlag: .DS.w 1


          ;; Detective -- Ende ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

          .END 


